% Copyright © 2017 Universitat Jaume I (UJI) and Tampere University of Technology (TUT)
%
% Permission is hereby granted, free of charge, to any person obtaining a copy of
% this software and associated documentation files (the "Software"), to deal in
% the Software without restriction, including without limitation the rights to
% use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
% of the Software, and to permit persons to whom the Software is furnished to do
% so, subject to the following conditions:
%
% The above copyright notice and this permission notice shall be included in all
% copies or substantial portions of the Software.
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
% OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
% SOFTWARE.

function [ results ] = knn_baseline( database_orig , k, defaultNonDetectedValue, newNonDetectedValue )
% knn_ips Indoor Positioning System based on kNN
%   
% inputs: 
%    database_orig           : database with RSSI values and Labels
%    k                       : value of nearest neighbors in kNN algoritm
%    datarep                 : Data representation
%    defaultNonDetectedValue : Value in DB for non detected RSSI values
%    newNonDetectedValue     : New Value for non detected RSSI values
%    distanceMetric          : Distance metric used for FP matching
%
% output:
%    results                 : output statistics
%
% Developed by J. Torres-Sospedra,
% Instiute of New Imaging Technologies, Universitat Jaume I
% jtorres@uji.es

tic;
% Remap bld numbers to 1:nblds

if size(database_orig.trainingLabels,2) == 2 %% Single-bld flr database
  database_orig.trainingLabels(:,3:5) = 0;
  database_orig.testLabels(:,3:5) = 0;
end

origBlds  = unique((database_orig.trainingLabels(:,5))); 
nblds     = size(unique(origBlds),1);
database0 = datarep_remapBldDB(database_orig,origBlds',1:nblds);

% Remap floors numbers to 1:nfloors
origFloors = unique((database_orig.trainingLabels(:,4))); 
nfloors    = size(unique(origFloors),1);
database0  = datarep_remapFloorDB(database0,origFloors',1:nfloors);

% Calculate the overall min RSSI (minus 1)
if size(newNonDetectedValue,1) == 0
newNonDetectedValue = min(...
                     [database0.trainingMacs(:)',...                      
                      database0.testMacs(:)']...
                     )-1;
end

% Replace default NonDetectedValue by the
database0.trainingMacs = database0.trainingMacs.*(database0.trainingMacs~=defaultNonDetectedValue)+newNonDetectedValue.*(database0.trainingMacs==defaultNonDetectedValue);
database0.testMacs     = database0.testMacs    .*(database0.testMacs~=defaultNonDetectedValue)    +newNonDetectedValue.*(database0.testMacs==defaultNonDetectedValue);

% Convert the RSSI to the Positive representation
minValue                = min([database0.trainingMacs(:)',database0.testMacs(:)']);
database.trainingMacs   = database0.trainingMacs-minValue;
database.testMacs       = database0.testMacs-minValue;
database.trainingLabels = database0.trainingLabels;
database.testLabels     = database0.testLabels;	
clear database0;
   
% Get number of samples
rsamples = size(database.trainingMacs,1); 
osamples = size(database.testMacs,1);
nmacs    = size(database.testMacs,2);

% Prepare output vectors
results.error      = zeros(osamples,5);
results.prediction = zeros(osamples,5);
results.targets    = zeros(osamples,5);
results.candidates = cell(osamples,1);
results.ncandidates= zeros(osamples,1);
results.distances  = cell(osamples,1);
results.timesample = zeros(osamples,5);
results.samples    = zeros(osamples,5);

% Get the deviations values for Mahalanobis Distance
%if strcmp(distanceMetric,'distance_mahalanobis')
    %deviation   = std(database.trainingMacs);
    %deviationsq = (deviation+(10000*deviation==0)).^2;
%end

fprintf('\nRunning the %s algorithm with\n',mfilename)
fprintf('    database features      : [%d,%d,%d]\n',rsamples,osamples,nmacs)
fprintf('    k                      : %d\n',k)
fprintf('    defaultNonDetectedValue: %d\n',defaultNonDetectedValue)
fprintf('    newNonDetectedValue    : %d\n',newNonDetectedValue)

time00 = toc;

for i = 1:osamples    

    time01 = toc;
        
    ofp = database.testMacs(i,:);
    
    time02 = toc;
      
    % Optimized euclidean distance
    distances = sqrt(sum((database.trainingMacs-(ones(rsamples,1)*ofp)).^2,2))';     
    
    time03 = toc;
    
	% Sort distances
    [distancessort,candidates] = sort(distances);
    
    % Set the value of candidates to k (kNN)
    ncandidates = k;
    
    % Check if candidates ranked in positions higher than k have the same
    % distance in the feature space.
    while ncandidates < rsamples
       if abs(distancessort(ncandidates)-distancessort(ncandidates+1))<0.000000000001
           ncandidates = ncandidates+1;
       else
           break
       end
    end

    % Estimate the building from the ncandidates
    probBld = zeros(1,nblds);
    for bld = 1:nblds
        probBld(bld) = sum(database.trainingLabels(candidates(1:ncandidates),5)==bld);
    end
    [~,bld] = max(probBld);
    
    % Estimate the floor from the ncandidates
    probFloor = zeros(1,nfloors);
    for floor = 1:nfloors
        probFloor(floor) = sum( (database.trainingLabels(candidates(1:ncandidates),4)==floor).*...
                                (database.trainingLabels(candidates(1:ncandidates),5)==bld));
    end
    [~,floor] = max(probFloor);
    
    % Estimate the coordinates from the ncandidates that belong to the
    % estimated floor
    points = 0;
    point  = [0,0,0];
    for j = 1:ncandidates
        if (database.trainingLabels(candidates(j),4)==floor).*(database.trainingLabels(candidates(j),5)==bld)
            points = points + 1;
            point  = point + database.trainingLabels(candidates(j),1:3);
        end
    end
    point = point / points;
    
    time04 = toc;
    % Extract the real-world bld (revert bld remap)
    realWorldBld = datarep_remapVector(bld,1:nblds,origBlds');
    
    % Extract the real-world floor (revert floor remap)
    realWorldFloor = datarep_remapVector(floor,1:nfloors,origFloors');
    
    % Generate the results statistics
    results.error(i,1) = sqrt(sum((database.testLabels(i,1:2)-point(1:2)).^2)); % 2D Positioning error in m 
    results.error(i,2) = sqrt(sum((database.testLabels(i,1:3)-point(1:3)).^2)); % 3D Positioning error in m
    results.error(i,3) = abs(database.testLabels(i,3)-point(3));                    % Height difference in m
    results.error(i,4) = abs(realWorldFloor - database_orig.testLabels(i,4));       % Height difference in floors
    results.error(i,5) = (realWorldBld ~= database_orig.testLabels(i,5));           % Building error
    
    results.prediction(i,:) = [point,realWorldFloor,realWorldBld];                  % Predicted position [x,y,z,floor]
    results.targets(i,:)    = database_orig.testLabels(i,1:5);                      % Current position   [x,y,z,floor]
    results.ncandidates(i,1)= ncandidates;                                          % Number of nearest neighbours used to estimate the position
    results.candidates{i,1} = candidates(1:10);                                     % Number of nearest neighbours used to estimate the position
    results.distances{i,1}  = distancessort(1:10);                                  % Distance in feature space of the best match
    
    results.samples(i,1)    = rsamples;                % Number of training/reference samples computed
    results.timesample(i,1) = time01;                  % time 1: initial time
    results.timesample(i,2) = time02;                  % time 2: time after operational fp preprocessing
    results.timesample(i,3) = time03;                  % time 3: time after matching  
    results.timesample(i,4) = time04;                  % time 4: time after positioning  
    results.timesample(i,5) = toc;                     % time 5: ending time  
    
    
end

results.timefull = [time00,toc-time00];

fprintf('Successfully executed!\n')
fprintf('    Mean Positioning Error 2D*  : %3.3f\n',mean(results.error((results.error(:,4)==0),1)));  % Mean 2D Positioning error when estimate and true position are in the same floor
fprintf('    Mean Positioning Error 3D   : %3.3f\n',mean(results.error(:,2)));   
fprintf('    Mean Positioning Error EvAAL: %3.3f\n',mean(results.error(:,1)+4*results.error(:,4)+50*results.error(:,5)));                        % Mean 3D Positioning error
fprintf('    Mean Positioning Error IPIN : %3.3f\n',mean(results.error(:,1)+15*results.error(:,4)+50*results.error(:,5)));                        % Mean 3D Positioning error
fprintf('    Bld&Floor detection hit rate: %3.3f\n',mean((results.error(:,4)==0).*(results.error(:,5)==0))*100);                 % Floor detection rate
fprintf('    BD Preprocessing time       : %3.6f\n',time00);    
fprintf('    Average preprocessing time  : %3.6f\n',mean(results.timesample(:,2)-results.timesample(:,1)));    
fprintf('    Average matching time       : %3.6f\n',mean(results.timesample(:,3)-results.timesample(:,2)));    
fprintf('    Average positioning time    : %3.6f\n',mean(results.timesample(:,4)-results.timesample(:,3)));   
fprintf('    Full time                   : %3.6f\n',results.timefull(2)); 
fprintf('        DB Preprocessing        : %3.3f%%\n',results.timefull(1)/results.timefull(2)*100); 
fprintf('        FP Preprocessing        : %3.3f%%\n',sum(results.timesample(:,2)-results.timesample(:,1))/results.timefull(2)*100); 
fprintf('        Matching                : %3.3f%%\n',sum(results.timesample(:,3)-results.timesample(:,2))/results.timefull(2)*100); 
fprintf('        Positioning             : %3.3f%%\n',sum(results.timesample(:,4)-results.timesample(:,3))/results.timefull(2)*100); 
fprintf('        Looping                 : %3.3f%%\n\n',sum(results.timesample(:,5)-results.timesample(:,4))/results.timefull(2)*100); 