% Copyright © 2017 Universitat Jaume I (UJI) and Tampere University of Technology (TUT)
%
% Permission is hereby granted, free of charge, to any person obtaining a copy of
% this software and associated documentation files (the "Software"), to deal in
% the Software without restriction, including without limitation the rights to
% use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
% of the Software, and to permit persons to whom the Software is furnished to do
% so, subject to the following conditions:
%
% The above copyright notice and this permission notice shall be included in all
% copies or substantial portions of the Software.
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
% OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
% SOFTWARE.

clear; clc;

% set to 0 if matlab, set to 1 if octave
IS_OCTAVE = exist('OCTAVE_VERSION', 'builtin') ~= 0;

if IS_OCTAVE
    pkg load statistics
    pkg load optim
    more off;
end

fprintf('Starting Databases generation ...\n');	% This process is done to speed up further data loading 

generateDB_simdb(IS_OCTAVE);
generateDB_libdb_52(IS_OCTAVE);
generateDB_libdb_26(IS_OCTAVE);
generateDB_libdb_13_2(IS_OCTAVE);
generateDB_libdb_13_5(IS_OCTAVE);
generateDB_mandb(IS_OCTAVE);
generateDB_mandb_reduced(IS_OCTAVE);

fprintf('Executing experiments for database simdb ...\n');

run_normal('simdb',IS_OCTAVE);
for i=1:8;
	run_remove_antenna('simdb',i,IS_OCTAVE);
	run_remove_antennas_random('simdb',i,IS_OCTAVE);
	run_switch_antennas_random('simdb',i,IS_OCTAVE);
end

fprintf('Executing experiments for database libdb with 52 independent APs ...\n');

run_normal('libdb_52',IS_OCTAVE);
for i=1:52;
	run_remove_antenna('libdb_52',i,IS_OCTAVE);
	run_remove_antennas_random('libdb_52',i,IS_OCTAVE);
	run_switch_antennas_random('libdb_52',i,IS_OCTAVE);
end

fprintf('Executing experiments for database libdb with 13 APs broadcasting 4 networks each...\n');

for i=1:13;
    run_remove_physicalantenna('libdb_52',i,IS_OCTAVE);
	run_remove_physicalantennas_random('libdb_52',i,IS_OCTAVE);
	run_switch_physicalantennas_random('libdb_52',i,IS_OCTAVE);
end

% The following commented code was not finally used in the paper.

% run_normal('libdb_26',IS_OCTAVE);
% for i=1:26;
	% run_remove_antenna('libdb_26',i,IS_OCTAVE);
	% run_remove_antennas_random('libdb_26',i,IS_OCTAVE);
	% run_switch_antennas_random('libdb_26',i,IS_OCTAVE);
% end
% for i=1:13;
 % run_remove_physicalantenna('libdb_26',i,IS_OCTAVE);
	% run_remove_physicalantennas_random('libdb_26',i,IS_OCTAVE);
	% run_switch_physicalantennas_random('libdb_26',i,IS_OCTAVE);
% end

fprintf('Executing experiments for database libdb with 13 APs, each representing the combination of the two networks emitted at 2.4 GHz by the same device\n');

run_normal('libdb_13_2',IS_OCTAVE);
for i=1:13;
	run_remove_antenna('libdb_13_2',i,IS_OCTAVE);
	run_remove_antennas_random('libdb_13_2',i,IS_OCTAVE);
	run_switch_antennas_random('libdb_13_2',i,IS_OCTAVE);
end

fprintf('Executing experiments for database libdb with 13 APs, each representing the combination of the two networks emitted at 5 GHz by the same device\n');

run_normal('libdb_13_5',IS_OCTAVE);
for i=1:13;
	run_remove_antenna('libdb_13_5',i,IS_OCTAVE);
	run_remove_antennas_random('libdb_13_5',i,IS_OCTAVE);
	run_switch_antennas_random('libdb_13_5',i,IS_OCTAVE);
end

fprintf('Executing experiments for database mandb with all 28 independent APs ...\n');

run_normal('mandb',IS_OCTAVE);
for i=1:28;
   run_remove_antenna('mandb',i,IS_OCTAVE);
   run_remove_antennas_random('mandb',i,IS_OCTAVE);
   run_switch_antennas_random('mandb',i,IS_OCTAVE);
end

fprintf('Executing experiments for database mandb with the 11 APs whose location was provided...\n');

run_normal('mandb_reduced',IS_OCTAVE);
for i=1:11;
  run_remove_antenna('mandb_reduced',i,IS_OCTAVE);
  run_remove_antennas_random('mandb_reduced',i,IS_OCTAVE);
  run_switch_antennas_random('mandb_reduced',i,IS_OCTAVE);
end

fprintf('Executing spatial analysis experiments for database libdb with 13 APs, each representing the combination of the two networks emitted at 2.4 GHz by the same device\n');

k = 3;
run_normal_with_spatial_analysis('libdb_13_2', k,IS_OCTAVE);
for i=1:13;
   run_remove_antenna_with_spatial_analysis('libdb_13_2',i, k,IS_OCTAVE);
end

fprintf('Executing spatial analysis experiments for database mandb with the 11 APs whose location was provided\n');

k = 5;
run_remove_antenna_mandb_reduced_spatial_analysis(k,IS_OCTAVE);

fprintf('Generating figures ...');

figure01(IS_OCTAVE);	% Impact of basic AP manipulations in the distance to the closest match in the RSS space (using simdb)
figure02(IS_OCTAVE);	% Impact of removing all 8 APs on the distance to the closest match in the RSS space for two zones (using simdb)
figure03(IS_OCTAVE);	% Impact of randomly removing n APs on the average distance to the closest match in the RSS space (using libdb_52, libdb_13_2 and libdb_13_5)
figure04(IS_OCTAVE);	% Percentage of valid RSSs from AP6 at each reference point in the two floors, separated for the 2.4 GHz and 5.2 GHz band (using libdb_13_2 and libdb_13_5)
figure05(IS_OCTAVE);	% Impact of disconnecting a particular AP in the distance to the closest match in the RSS space and positioning error (using libdb)
figure06(IS_OCTAVE);	% Impact of swapping sets of APs in the distance to the closest match in the RSS space (using libdb_13_2 and libdb_13_5)
figure07(IS_OCTAVE);	% Impact of disabling a particular AP on the relative distribution of position estimates (using libdb_13_2)
% Figure 8 did not result from experiments
figure09(IS_OCTAVE);	% Impact of disconnecting n random APs in the distance in the RSS space to the closest match and positioning error (using mandb)
figure10(IS_OCTAVE);	% Impact of disconnecting a particular AP in the distance in the RSS space to the closest match and positioning error (using mandb_reduced)
figure11(IS_OCTAVE);	% Impact of disabling a particular AP on the relative distribution of position estimates (using mandb)
figure12(IS_OCTAVE);	% Impact of disconnecting a particular AP in the distance to the closest match in the RSS space in each zone (using mandb_reduced)
figure13(IS_OCTAVE);	% Relation between relative averaged positioning error and relative distance in the RSS distance  (using libdb_13_2 and mandb_reduced)

